#!/usr/bin/env python3

## autofitimages.py, Version 0.2

## Copyright (c) 2013 - 2022 Christian Zietz, <czietz@gmx.net>
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
## 
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
## 
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.

import sys
import re
import os
from PIL import Image

if len(sys.argv)<2:
    print("Call %s with LaTeX logfile as first argument" % sys.argv[0])
    sys.exit(2)

try:
    DPI = int(sys.argv[2])
except:
    DPI = 600
try:
    JPEGQUAL = int(sys.argv[3])
except:
    JPEGQUAL = 90
AUTOFITDIR = "autofit"
DONTFIT = ".dontfit"
GOTBIGGER = ".gotbigger"

def getmtime(path):
    try:
        return os.path.getmtime(path)
    except:
        return 0

def tryremove(path):
    try:
        os.remove(path)
    except:
        pass

def trymkdir(path):
    try:
        os.mkdir(path)
    except:
        pass

def touch(path):
    if os.path.exists(path):
        os.utime(path,None)
    else:
        open(path,"w").close()

try:
    with open(sys.argv[1],"rt", newline=None) as log:
        texlogs = log.read()
except:
    print("Could not read logfile: " + sys.argv[1])
    sys.exit(1)

# All paths are relative to the directory of the logfile
os.chdir(os.path.dirname(os.path.abspath(sys.argv[1])))
    
print("Parsing " + sys.argv[1])
print("DPI = {0}, JPEG = {1}".format(DPI, JPEGQUAL))

filedict = {}
for match in re.finditer("^\s*<use (.*?)>$.*?Requested size: ([0-9.]+)pt x ([0-9.]+)pt",texlogs,re.MULTILINE | re.DOTALL):
    nw = int(float(match.group(2)) / 72.27 * DPI + .5)
    nh = int(float(match.group(3)) / 72.27 * DPI + .5)
    
    fn = os.path.abspath(match.group(1))
    ext = os.path.splitext(fn)[1].upper()

    # These are the file type currently supported
    if (ext != '.JPG') and (ext != '.PNG') and (ext != '.JPEG'):
        continue
    
    if os.path.normcase(os.path.basename(os.path.dirname(fn))) == os.path.normcase(AUTOFITDIR):
        destfn = fn
        srcfn = os.path.join(os.path.dirname(os.path.dirname(fn)),os.path.basename(fn))
    else:
        srcfn = fn
        destfn = os.path.join(os.path.dirname(fn),AUTOFITDIR,os.path.basename(fn))
        # Create autofit directory if it doesn't exist
        trymkdir(os.path.join(os.path.dirname(fn),AUTOFITDIR))
    
    if srcfn in filedict:
        (num, srcfn, destfn, ow, oh) = filedict[srcfn]
        filedict[srcfn] = (num, srcfn, destfn, max(nw,ow), max(nh,oh))
    else:
        filedict[srcfn] = (len(filedict), srcfn, destfn, nw, nh)

# File list sorted by first occurence as the first item is the index
for (num, srcfn, destfn, nw, nh) in sorted(filedict.values()):
    print("")
    print("Source: " + srcfn)
    print("Dest:   " + destfn)
    print("Size:   " + "{0} x {1}".format(nw,nh))
    
    if os.path.exists(srcfn + DONTFIT):
        print("Autofitting disabled")
        tryremove(destfn)
        tryremove(destfn + GOTBIGGER)
        continue
    
    if not os.path.exists(srcfn):
        print("Source file doesn't exist")
        continue

    # True if destfn (or corresponding .gotbigger file) exists and is newer than source file    
    srcunchanged = getmtime(srcfn) < max(getmtime(destfn),getmtime(destfn + GOTBIGGER))
    
    try:
        im = Image.open(destfn)
        (iw, ih) = im.size
        del im
    except:
        iw = 0
        ih = 0
        
    sizeunchanged = ((iw == nw) and (ih == nh))
    
    if srcunchanged and sizeunchanged:
        print("Source file and print size unchanged")
        continue
    
    try:
        im = Image.open(srcfn)
        imformat = im.format
        im = im.resize((nw,nh), resample=Image.Resampling.LANCZOS)
        if imformat == "JPEG":
            im.save(destfn, imformat, quality=JPEGQUAL, dpi=(DPI,DPI))
        elif imformat == "PNG":
            im.save(destfn, imformat, optimize=True, dpi=(DPI,DPI))
        else: # this should not happen anyway
            im.save(destfn, imformat, dpi=(DPI,DPI))
        del im
    except:
        print("Error while creating image")
        tryremove(destfn)
        tryremove(destfn + GOTBIGGER)
        continue
    
    if os.path.getsize(destfn) >= os.path.getsize(srcfn):
        print("Fitted image is bigger")
        tryremove(destfn)
        touch(destfn + GOTBIGGER)
    else:
        print("OK!")
        tryremove(destfn + GOTBIGGER)
